import seaborn as sns
import matplotlib.pyplot as plt
import pandas as pd

# load data from output csv file
file_path = 'output.csv'
df = pd.read_csv(
    file_path,
    delimiter=',',
    on_bad_lines='skip'
)

# Remove unnessary rows
df_valid_places = df[(df['destination'] != 'O-O-O') & (df['destination'] != 'O-O')]

def getDf(df: pd.DataFrame, player: str, isHuman: bool) -> {pd.DataFrame}:
    return df[(df['player'] == player) & (df['isHuman'] == isHuman)].reset_index(drop=True).drop(columns=['player', 'isHuman'])

def make2DMap(df: pd.DataFrame) -> {pd.DataFrame}:
    df['col'] = df['destination'].str[0]
    df['row'] = df['destination'].str[1].astype(int)
    return df.drop(columns=['destination'])

def makeFigure(df: pd.DataFrame, color: str, type: str):
    plt.figure(figsize=(10, 8))
    sns.heatmap(df, annot=True, fmt=".2f", cmap="coolwarm", cbar=True, linewidths=0.5)
    plt.title(f'2D Heatmap of avg(dcm) for [color={color}, type={type}]', fontsize=16)
    plt.xlabel('Column (1-8)')
    plt.ylabel('Row (a-h)')

def getType(type: bool) -> {str}:
    if (type == True):
        return 'human'
    else: 
        return 'computer'

def processData(df: pd.DataFrame, color: str, isHuman: bool):
    df_filtered = getDf(df, color, isHuman)
    twoD_map = make2DMap(df_filtered)
    pivot_map = twoD_map.pivot(index='row', columns='col', values='avg(dcm)').sort_index(ascending=False).sort_index(axis=1)
    makeFigure(pivot_map, color, getType(isHuman))

processData(df_valid_places, 'white', True)
processData(df_valid_places, 'white', False)
processData(df_valid_places, 'black', True)
processData(df_valid_places, 'black', False)

plt.show()