# Import all libraries
import time
from pyspark.sql import SparkSession
from pyspark.sql.functions import col
import numpy as np

spark = SparkSession.builder.appName('BLUNDER').getOrCreate()

# A lot of files quickly become too much to merch, so we iterate over them 1 by 1
years= np.arange(23, 24) # range of years
months = np.arange(12, 13) # range of months
adj_months = []
means = [] # Here we will store the results of 1 month
all_res=[] # Here we will store the results of all months
moves_ahead = 5 # How many moves after the blunder we evaluate
for month in months: # Make sure the strings have the right format for iteration
    month = str(month)
    if len(month) ==1:
        month = '0'+month
    adj_months.append(month)


for year in years:
    for month in adj_months:
        means=[]
        # Load in file
        path = '/data/doina/Lichess/lichess_db_standard_rated_20' + str(year) + '-' + str(month) + '.parquet'
        # We start of with a data frame
        df = spark.read.parquet(path)
        # Filter Everything that is not rapid or classical and make sure all games contain eval data and blunders
        df2 = df.filter(col("Event").contains('Rapid') | col("Event").contains('Classical'))
        df3 = df2.filter(col("Moves").contains('eval') | col("Moves").contains('??') )
        # For now it seemed practical to classify the games by average rating, even though not technically nescesarry, since we can filter on black and white and use these rating, but then we would be evaluating games without the moves of the other player
        df4 = df3.withColumn("AverageRating", (df3.BlackElo+df3.WhiteElo)/2)
        # Remove unnescesarry columns
        df5 = df4.select(col('Event'), col('Moves'), col('AverageRating'))

        # Filter the data in different rating groups
        df_beginner = df5.filter(col('AverageRating')<1000)
        df_intermediate = df5.filter((col('AverageRating') >= 1000) & (col('AverageRating') < 1400))
        df_advanced = df5.filter((col('AverageRating') >= 1400) & (col('AverageRating') < 2000))
        df_master = df5.filter(col('AverageRating') >= 2000)

        # Iterate over these dataframes 
        dataframes = [df_beginner, df_intermediate, df_advanced, df_master]
        for dataframe in dataframes:
            # We seek the format [move number, move, evaluation]
            # We turn the dataframe into an rdd, to expend the rows with the lenght of the games
            rdd1 = dataframe.rdd
            rdd2 = rdd1.map(lambda t: t[1].split('}')) \
                .map(lambda t: [[s.split('] [')[0] for s in t[:-2]], t[-1].replace(' ', '')] if '%c' in t[0] else [t[:-2], t[-1].replace(' ', '')]) \
                .map(lambda t: [[s.split(' { [%eval ') for s in t[0]], t[1]]) \
                .filter(lambda t: len(t)==2 and all(len(s)==2 for s in t[0])) \
                .map(lambda t: [[[s[0].replace(' ', ''), s[1]] for s in t[0]], t[1]]) \
                .map(lambda t: [[[s[0].split('.')[0], s[0].split('.')[-1], s[1]] for s in t[0]], t[1]]) \
                .map(lambda t: [list(zip(np.arange(len(t[0])), t[0])),t[1]]) \
                .map(lambda t: [[[int(s[0]), s[1][1], s[1][2]] for s in t[0]], t[1]]) \
                .map(lambda t: [[s if '#' not in s[2] else ([s[0], s[1], 20] if float(s[2][1:]) > 0 else [s[0], s[1], -20]) for s in t[0]], t[1]]) \
                .map(lambda t: [t[0], min([s[0] if '??' in s[1] else 2000 for s in t[0]]), t[1]]) \
                .filter(lambda t: not t[1] == 2000) \
                .map(lambda t: ( t[0] + [[t[1] + i + 1,'#', 20 if t[2] == "1-0" else -20 if t[2] == "0-1" else 0] for i in range(moves_ahead*2)], t[1], t[2] )) \
                .map(lambda t: [float(t[0][t[1]-1][2]), float(t[0][t[1]][2]), [float(t[0][i][2]) for i in range(t[1] + 1, t[1] + moves_ahead*2, 2)]]) \
                .map(lambda t: [t[1]-t[0], [s-t[0] for s in t[2]]]) \
                .filter(lambda t: not t[0] == 0) \
                .map(lambda t: [s/t[0] for s in t[1]])

                # Explanation
                # .map(lambda t: t[1].split('}')) \
                # Removes everything but the game and splits it up into individual moves
                # 1 move = '1. e4 { [%eval 0.2] [%clk 0:10:00] '
                
                # .map(lambda t: [[s.split('] [')[0] for s in t[:-2]], t[-1].replace(' ', '')] if '%c' in t[0] else [t[:-2], t[-1].replace(' ', '')]) \
                # Split the clock off, if it is there
                # '1. e4 { [%eval 0.2'
                
                # .map(lambda t: [[s.split(' { [%eval ') for s in t[0]], t[1]]) \
                # Split the move from the eval
                # ['1. e4', '0.2']
                
                # .filter(lambda t: len(t)==2 and all(len(s)==2 for s in t[0])) \
                # Make sure the next function works
                
                # .map(lambda t: [[[s[0].replace(' ', ''), s[1]] for s in t[0]], t[1]]) \
                # Remove the spaces from the move
                # ['1.e4', '0.2']
                
                # .map(lambda t: [[[s[0].split('.')[0], s[0].split('.')[-1], s[1]] for s in t[0]], t[1]]) \
                # split the move number from the move
                # ['1', 'e4', '0.2']
                
                # .map(lambda t: [list(zip(np.arange(len(t[0])), t[0])),t[1]]) \
                # White and black now will have the same move number as the moves go 1,1,2,2,3,3 This changes that
                # (np.int64(0), ['1', 'e4', '0.2'])
                
                # .map(lambda t: [[[int(s[0]), s[1][1], s[1][2]] for s in t[0]], t[1]]) \
                # Combine the formats
                # [0, 'e4', '0.2']
                
                # .map(lambda t: [[s if '#' not in s[2] else ([s[0], s[1], 20] if float(s[2][1:]) > 0 else [s[0], s[1], -20]) for s in t[0]], t[1]]) \
                # if a move has a #, it is a checkmate in n evaluation, if advantage to white it becomes +20, if advantage to black, it becomes -20
                
                # .map(lambda t: [t[0], min([s[0] if '??' in s[1] else 2000 for s in t[0]]), t[1]]) \
                # Append the move of the first blunder to the set, if this does not exist, append 2000
                # Format is now [game, move of first blunder, final result]
                
                # .filter(lambda t: not t[1] == 2000) \
                # Remove all games where we had to append 2000
                
                # .map(lambda t: ( t[0] + [[t[1] + i + 1,'#', 20 if t[2] == "1-0" else -20 if t[2] == "0-1" else 0] for i in range(moves_ahead*2)], t[1], t[2] )) \
                # This extends the game based on the final result, +20 or -20 against based on advantage and 0 if it has become a draw
                # If black has advantage, it extends the game with [59, '#', -20], [60, '#', -20],
                
                # .map(lambda t: [float(t[0][t[1]-1][2]), float(t[0][t[1]][2]), [float(t[0][i][2]) for i in range(t[1] + 1, t[1] + moves_ahead*2, 2)]]) \
                # save the eval before the blunder, after the blunder and the evals for a couple of moves ahead from 1 player
                # [[-6.54, -20.0, [-6.43, -6.47, -6.38, -6.87, -6.7]]]
                
                # .map(lambda t: [t[1]-t[0], [s-t[0] for s in t[2]]]) \
                # This stores the total advantage gained from the blunder and removes the advantage before the blunder to be able to calculate percentages ofver the whole
                # [-13.46, [0.11000000000000032, 0.07000000000000028, 0.16000000000000014, -0.33000000000000007, -0.16000000000000014]]
                
                # .filter(lambda t: not t[0] == 0) \
                # If there was a blunder, this usually is not 0, but we make sure the code works
                
                # .map(lambda t: [s/t[0] for s in t[1]])
                # Now calculate the advantage maintained
                # [-0.008172362555720677, -0.005200594353640437, -0.01188707280832096, 0.024517087667161964, 0.01188707280832096]

            # Save all the means and standard deviations
            res_means = [rdd2.map(lambda t: t[i]).mean() for i in range(len(rdd2.first()))]
            res_stds = [rdd2.map(lambda t: t[i]).stdev() for i in range(len(rdd2.first()))]

            # All the data we need for analysis
            means.append([year,month,rdd2.count(),res_means, res_stds])                                                                                                                                                                      
            print('')
            print('')
            print('')
            print(year, month) # keeps track of where we are when we run the code for longer
            print('')
            print('')
            print('')
            time.sleep(5)
        all_res.append([means])
# after all the executions have finished we print the results to the screen
print(all_res)