from pyspark import SparkContext
from pyspark.sql import SparkSession
from pyspark.sql.functions import col, arrays_zip, explode, lit, median

sc = SparkContext(appName="median_dcm_metric")
spark = SparkSession.builder.appName("median_dcm_metric").getOrCreate()


metrics = spark.read.parquet('hdfs:///user/s2755572/Chess_Metrics/Lichess_202312_CCRL_1990030/') \
    .filter((col('WhiteElo') > 2000) & (col('BlackElo') > 2000)) # Filter out games where at least one player is not a master
dcm_destination_white_pairs = metrics.select( col('WhiteMetrics.DCM.destination').alias('destination'),
                                              col('WhiteMetrics.DCM.dcm').alias('dcm'),
                                              col('isHuman')) \
    .withColumn('zipped', arrays_zip('destination', 'dcm')) \
    .withColumn('pairs', explode(col('zipped'))) \
    .withColumn('player', lit('white')) \
    .select(col('pairs'), col('isHuman'), col('player'))

dcm_destination_black_pairs = metrics.select( col('BlackMetrics.DCM.destination').alias('destination'),
                                              col('BlackMetrics.DCM.dcm').alias('dcm'),
                                              col('isHuman')) \
    .withColumn('zipped', arrays_zip('destination', 'dcm')) \
    .withColumn('pairs', explode(col('zipped'))) \
    .withColumn('player', lit('black')) \
    .select(col('pairs'), col('isHuman'), col('player'))

white_median_dcm_destination = dcm_destination_white_pairs \
    .select(col('pairs.destination').alias('destination'), col('pairs.dcm').alias('dcm'), col('isHuman'), col('player')) \
    .groupBy('destination', 'isHuman', 'player').agg(median('dcm')).alias('average_dcm') \
    .filter(col('destination') != 'null')

black_median_dcm_destination = dcm_destination_black_pairs \
    .select(col('pairs.destination').alias('destination'), col('pairs.dcm').alias('dcm'), col('isHuman'), col('player')) \
    .groupBy('destination', 'isHuman', 'player').agg(median('dcm')).alias('average_dcm') \
    .filter(col('destination') != 'null')

avg_dcm_destination = white_median_dcm_destination.union(black_median_dcm_destination)
#This is guaranteed to be small data, so we can save it to a the NFS instead
"""
The reason behind repartition vs coalesce here is stated in the documentation:
`However, if you’re doing a drastic coalesce, e.g. to numPartitions = 1, this may result in your computation taking place on fewer nodes than you like
(e.g. one node in the case of numPartitions = 1). To avoid this, you can call repartition(). This will add a shuffle step, but means the current upstream
partitions will be executed in parallel (per whatever the current partitioning is).`
"""
avg_dcm_destination.repartition(1).write.csv(path='file:///home/s2755572/Chess_Metrics_Output/Lichess_202312_CCRL_1990030/median/masters', mode='overwrite', header=True)