from pyspark import SparkContext
from pyspark.sql import SparkSession
from pyspark.sql.functions import col, split, concat_ws, collect_list

sc = SparkContext(appName="parse_CCRL_RDD_to_CSV")
spark = SparkSession.builder.appName("parse_CCRL_RDD_to_CSV").getOrCreate()

PATH = "file:///home/s2755572/data/CCRL-4040-1990030.pgn.tar.bz2"

EVENT_REGEX_PATTERN = '\[Event ".+"]'
SITE_REGEX_PATTERN = '\[Site ".+"]'
DATE_REGEX_PATTERN = '\[Date ".+"]'
ROUND_REGEX_PATTERN = '\[Round ".+"]'
WHITE_REGEX_PATTERN = '\[White ".+"]'
BLACK_REGEX_PATTERN = '\[Black ".+"]'
RESULT_REGEX_PATTERN = '\[Result ".+"]'
ECO_REGEX_PATTERN = '\[ECO ".+"]'
OPENING_REGEX_PATTERN = '\[Opening ".+"]'
VARIATION_REGEX_PATTERN = '\[Variation ".+"]'
PLYCOUNT_REGEX_PATTERN = '\[PlyCount ".+"]'
WHITEELO_REGEX_PATTERN = '\[WhiteElo ".+"]'
BLACKELO_REGEX_PATTERN = '\[BlackElo ".+"]'

df = spark.read.csv("hdfs:///user/s2755572/Computer_Chess_1990030/*") \
    .select(col('_c0').alias('game_idx'), col('_c1').alias('data')) \
    .dropna()

# Filter per attribute
events = df.filter(col('data').rlike(EVENT_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('event'))
site = df.filter(col('data').rlike(SITE_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('site'))
date = df.filter(col('data').rlike(DATE_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('date'))
round = df.filter(col('data').rlike(ROUND_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('round'))
white = df.filter(col('data').rlike(WHITE_REGEX_PATTERN)).select(col('game_idx'),split(col('data'), '"')[1].alias('white'))
black = df.filter(col('data').rlike(BLACK_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('black'))
result = df.filter(col('data').rlike(RESULT_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('result'))
eco = df.filter(col('data').rlike(ECO_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('eco'))
opening = df.filter(col('data').rlike(OPENING_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('opening'))
variation = df.filter(col('data').rlike(VARIATION_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('variation'))
plycount = df.filter(col('data').rlike(PLYCOUNT_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('plycount'))
whiteelo = df.filter(col('data').rlike(WHITEELO_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('whiteelo'))
blackelo = df.filter(col('data').rlike(BLACKELO_REGEX_PATTERN)).select(col('game_idx'), split(col('data'), '"')[1].alias('blackelo'))
game = df.filter(~col('data').startswith('[')).groupBy(col('game_idx')).agg(concat_ws(" ", collect_list('data')).alias('data'))

# Add all attributes to the same dataframe
test_df = events.join(site, 'game_idx', 'left') \
    .join(date, 'game_idx', 'left') \
    .join(round, 'game_idx', 'left') \
    .join(white, 'game_idx', 'left') \
    .join(black, 'game_idx', 'left') \
    .join(result, 'game_idx', 'left') \
    .join(eco, 'game_idx', 'left') \
    .join(opening, 'game_idx', 'left') \
    .join(variation, 'game_idx', 'left') \
    .join(plycount, 'game_idx', 'left') \
    .join(whiteelo, 'game_idx', 'left') \
    .join(blackelo, 'game_idx', 'left') \
    .join(game, 'game_idx', 'left')
    
test_df = test_df.repartition(20, 'game_idx')
test_df.write.csv(path='hdfs:///user/s2755572/Computer_Chess_1990030_Parsed', mode='overwrite', header=True)




