from pyspark import SparkContext
from pyspark.sql import SparkSession
from pyspark.sql.functions import col

sc = SparkContext(appName="parse_CCRL_into_RDD")
spark = SparkSession.builder.appName("parse_CCRL").getOrCreate()

PATH = "file:///home/s2755572/data/CCRL-4040-1990030.pgn.tar.bz2"

game_number = 0

rdd = sc.textFile(PATH)

def filter_per_attribute(df, attribute):
    return df.filter(col('data').rlike(attribute)).select(col('game_idx'), col('data').alias(attribute))

# Python globals don't work, but we can use the partition index to keep track of each partition.
# In this way, each game will receive a unique ID regardless of the number of executor nodes.
# ID format will be gameNumber_partitionIdx.
def add_game_index(partitionIdx, row):
    global game_number
    if row.startswith('[Event '):
        game_number += 1
    return (str(game_number) + '_' + str(partitionIdx), row)

games = rdd.mapPartitionsWithIndex(lambda index, iterator: map(lambda row: add_game_index(index, row) ,iterator)) \
    .map(lambda row: str(row[0] + ',' + row[1]))
games.saveAsTextFile("hdfs:///user/s2755572/Computer_Chess_1990030")




