import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import re

# Load the data
data = pd.read_csv('output.csv')

data = data[~data['destination'].isin(['O-O', 'O-O-O'])]


# Define a regular expression pattern for valid chess squares
valid_chess_square_pattern = r'^[a-h][1-8]$'

# Filter valid chess squares
data = data[data['destination'].str.match(valid_chess_square_pattern)]

#Human games and select color
data = data[data['isHuman']]
data = data[data['player']=='white']

aggregated_data = data.groupby('destination')['avg(dcm)'].mean().reset_index()

oldData =data.copy()

data=aggregated_data

# Initialize an 8x8 chessboard matrix with NaN values
chessboard = np.full((8, 8), np.nan)

# Map chessboard letters to indices
col_mapping = {'a': 0, 'b': 1, 'c': 2, 'd': 3, 'e': 4, 'f': 5, 'g': 6, 'h': 7}

# Fill the matrix with avg(dcm) values
for _, row in data.iterrows():
    col = col_mapping[row['destination'][0]]  # Extract column (letter) index
    row_idx = 8 - int(row['destination'][1])  # Extract row (number) index
    chessboard[row_idx, col] = row['avg(dcm)']

# Create the heatmap
plt.figure(figsize=(8, 8))
sns.heatmap(chessboard, annot=True, fmt=".2f", cmap="coolwarm", linewidths=0.5,
            xticklabels=list(col_mapping.keys()), yticklabels=list(range(8, 0, -1)))

plt.title("Chessboard Heatmap (Evaluation Changes) player white human games")
plt.xlabel("Columns")
plt.ylabel("Rows")
plt.show()